@@ -77,7 +77,6 @@ error:
 	*sockp = NULL;
 	return err;
 }
-EXPORT_SYMBOL(udp_sock_create4);
 
 void setup_udp_tunnel_sock(struct net *net, struct socket *sock,
 			   struct udp_tunnel_sock_cfg *cfg)
@@ -85,9 +84,10 @@ void setup_udp_tunnel_sock(struct net *net, struct socket *sock,
 	inet_sk(sock->sk)->mc_loop = 0;
 	encap_rcv = cfg->encap_rcv;
 	rcu_assign_sk_user_data(sock->sk, cfg->sk_user_data);
-	sock->sk->sk_data_ready = our_sk_data_ready;
+	/* We force the cast in this awful way, due to various Android kernels
+	 * backporting things stupidly. */
+	*(void **)&sock->sk->sk_data_ready = (void *)our_sk_data_ready;
 }
-EXPORT_SYMBOL_GPL(setup_udp_tunnel_sock);
 
 #if LINUX_VERSION_CODE < KERNEL_VERSION(3, 16, 0)
 static inline __sum16 udp_v4_check(int len, __be32 saddr,
@@ -141,6 +141,7 @@ static void our_iptunnel_xmit(struct rtable *rt, struct sk_buff *skb,
 		  __u8 tos, __u8 ttl, __be16 df, bool xnet)
 {
 	struct iphdr *iph;
+	struct pcpu_tstats *tstats = this_cpu_ptr(skb->dev->tstats);
 
 	skb_scrub_packet(skb, xnet);
 
@@ -169,6 +170,9 @@ static void our_iptunnel_xmit(struct rtable *rt, struct sk_buff *skb,
 #endif
 
 	iptunnel_xmit(skb, skb->dev);
+	u64_stats_update_begin(&tstats->syncp);
+	tstats->tx_bytes -= 8;
+	u64_stats_update_end(&tstats->syncp);
 }
 #define iptunnel_xmit our_iptunnel_xmit
 #endif
@@ -209,10 +213,10 @@ void udp_tunnel_xmit_skb(struct rtable *rt, struct sock *sk, struct sk_buff *skb
 #endif
 			   rt, skb, src, dst, IPPROTO_UDP, tos, ttl, df, xnet);
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(3, 12, 0)
-	iptunnel_xmit_stats(ret, &dev->stats, dev->tstats);
+	if (ret)
+		iptunnel_xmit_stats(ret - 8, &dev->stats, dev->tstats);
 #endif
 }
-EXPORT_SYMBOL_GPL(udp_tunnel_xmit_skb);
 
 void udp_tunnel_sock_release(struct socket *sock)
 {
@@ -220,7 +224,6 @@ void udp_tunnel_sock_release(struct socket *sock)
 	kernel_sock_shutdown(sock, SHUT_RDWR);
 	sock_release(sock);
 }
-EXPORT_SYMBOL_GPL(udp_tunnel_sock_release);
 
 #if IS_ENABLED(CONFIG_IPV6)
 #include <linux/module.h>
@@ -296,7 +299,6 @@ error:
 	*sockp = NULL;
 	return err;
 }
-EXPORT_SYMBOL_GPL(udp_sock_create6);
 
 #if LINUX_VERSION_CODE < KERNEL_VERSION(3, 16, 0)
 static inline __sum16 udp_v6_check(int len,
@@ -382,5 +384,4 @@ int udp_tunnel6_xmit_skb(struct dst_entry *dst, struct sock *sk,
 	ip6tunnel_xmit(skb, dev);
 	return 0;
 }
-EXPORT_SYMBOL_GPL(udp_tunnel6_xmit_skb);
 #endif
